@@ -1,7 +1,8 @@
 import json
+import os
+import re
 from pathlib import Path
 from datetime import datetime
-from datetime import timedelta
 
 import hcl2
 from click.exceptions import Exit
@@ -21,6 +22,15 @@ from leverage.path import get_account_config_path
 from leverage.path import NotARepositoryError
 from leverage.conf import load as load_env
 
+REGION = (
+    r"(.*)"  # project folder
+    # start region 
+    r"(global|(?:[a-z]{2}-(?:gov-)?"
+    r"(?:central|north|south|east|west|northeast|northwest|southeast|southwest|secret|topsecret)-[1-4]))"
+    # end region
+    r"(.*)"  # layer
+)
+
 
 def get_docker_client():
     """ Attempt to get a Docker client from the environment configuration. Halt application otherwise.
@@ -91,7 +101,7 @@ class LeverageContainer:
         self.image_tag = self.env_conf.get("TERRAFORM_IMAGE_TAG")
         if not self.image_tag:
             logger.error("No docker image tag defined.\n"
-                         "Please set `TERRAFORM_IMAGE_TAG` variable before running a Leverage command.")
+                         "Please set `TERRAFORM_IMAGE_TAG` variable in the project's [bold]build.env[/bold] file before running a Leverage command.")
             raise Exit(1)
 
         # Get project name
@@ -167,6 +177,18 @@ class LeverageContainer:
     def guest_aws_credentials_dir(self):
         return f"/root/tmp/{self.project}"
 
+    @property
+    def region(self):
+        """
+        Return the region of the layer.
+        """
+        if matches := re.match(REGION, self.cwd.as_posix()):
+            # the region (group 1) is between the projects folders (group 0) and the layers (group 2)
+            return matches.groups()[1]
+
+        logger.exception(f"No valid region could be found at: {self.cwd.as_posix()}")
+        raise Exit(1)
+
     def ensure_image(self):
         """ Make sure the required Docker image is available in the system. If not, pull it from registry. """
         found_image = self.client.api.images(f"{self.image}:{self.image_tag}")
@@ -269,6 +291,20 @@ class LeverageContainer:
 
         return self._run(container, run_func)
 
+    def _start_with_output(self, command="/bin/bash", *args):
+        """
+        Same than _start but also returns the outputs (by dumping the logs) of the container.
+        """
+        container = self._create_container(True, command, *args)
+
+        def run_func(client, container):
+            dockerpty.start(client=client.api, container=container)
+            exit_code = client.api.inspect_container(container)["State"]["ExitCode"]
+            logs = client.api.logs(container).decode("utf-8")
+            return exit_code, logs
+
+        return self._run(container, run_func)
+
     def start(self, command="/bin/sh", *arguments):
         """ Run command with the given arguments in an interactive container.
 
@@ -406,6 +442,9 @@ class TerraformContainer(LeverageContainer):
         self.sso_enabled = self.common_conf.get("sso_enabled", False)
         self.mfa_enabled = self.env_conf.get("MFA_ENABLED", "false") == "true" # TODO: Convert values to bool upon loading
 
+        # SSH AGENT
+        SSH_AUTH_SOCK = os.getenv('SSH_AUTH_SOCK')
+
         self.environment = {
             "COMMON_CONFIG_FILE": self.common_tfvars,
             "ACCOUNT_CONFIG_FILE": self.account_tfvars,
@@ -417,15 +456,26 @@ class TerraformContainer(LeverageContainer):
             "AWS_CACHE_DIR": f"{self.guest_aws_credentials_dir}/cache",
             "SSO_CACHE_DIR": f"{self.guest_aws_credentials_dir}/sso/cache",
             "SCRIPT_LOG_LEVEL": get_script_log_level(),
-            "MFA_SCRIPT_LOG_LEVEL": get_script_log_level() # Legacy
+            "MFA_SCRIPT_LOG_LEVEL": get_script_log_level(),  # Legacy
+            "SSH_AUTH_SOCK": '' if SSH_AUTH_SOCK is None else '/ssh-agent',
         }
         self.entrypoint = self.TF_BINARY
         self.mounts = [
             Mount(source=self.root_dir.as_posix(), target=self.guest_base_path, type="bind"),
             Mount(source=self.host_aws_credentials_dir.as_posix(), target=self.guest_aws_credentials_dir, type="bind"),
-            Mount(source=(self.home / ".ssh").as_posix(), target="/root/.ssh", type="bind"),
-            Mount(source=(self.home / ".gitconfig").as_posix(), target="/etc/gitconfig", type="bind")
+            Mount(source=(self.home / ".gitconfig").as_posix(), target="/etc/gitconfig", type="bind"),
         ]
+        # if you have set the tf plugin cache locally
+        if tf_cache_dir := os.getenv("TF_PLUGIN_CACHE_DIR"):
+            # then mount it too into the container
+            self.environment["TF_PLUGIN_CACHE_DIR"] = tf_cache_dir
+            # given that terraform use symlinks to point from the .terraform folder into the plugin folder
+            # we need to use the same directory inside the container
+            # otherwise symlinks will be broken once outside the container
+            # which will break terraform usage outside Leverage
+            self.mounts.append(Mount(source=tf_cache_dir, target=tf_cache_dir, type="bind"))
+        if SSH_AUTH_SOCK is not None:
+            self.mounts.append(Mount(source=SSH_AUTH_SOCK, target="/ssh-agent", type="bind"))
 
         self._backend_key = None
 
@@ -460,8 +510,7 @@ class TerraformContainer(LeverageContainer):
                                for common_file in self.common_config_dir.glob("*.tfvars")]
         account_config_files = [f"-var-file={self._guest_config_file(account_file)}"
                                 for account_file in self.account_config_dir.glob("*.tfvars")]
-        region_settings      = [f"-var=\"region={self.region}\""]
-        return common_config_files + account_config_files + region_settings
+        return common_config_files + account_config_files
 
     def enable_mfa(self):
         """ Enable Multi-Factor Authentication. """
@@ -580,53 +629,6 @@ class TerraformContainer(LeverageContainer):
         self._prepare_container()
         self._start()
 
-    @property
-    def region(self):
-        """ Determine the region based on PATH, account tfvars or project tfvars. Returns a string with the region name. """
-        region = self._find_region(self.cwd.relative_to(self.account_dir))
-
-        if not region is None:
-            return region
-
-        logger.error("No region found")
-        raise Exit(1)
-
-    @property
-    def terraform_backend(self):
-        """ Determine the terraform backend for the account. Returns an object with backend information. """
-        # These are the possible backend layer names for Refarch v1 and v2
-        possible_layer_names = ['terraform-backend', 'base-tf-backend']
-        for layer_name in possible_layer_names:
-            for directory in Path(self.account_dir).glob(f"**/{layer_name}"):
-
-                region = self._find_region(Path(directory).relative_to(self.account_dir))
-
-                if not region is None:
-                    return {'directory': directory , 'region': region}
-                else:
-                    logger.error("No region found for backend layer")
-                    raise Exit(1)
-
-        logger.error("No backend layer found")
-        raise Exit(1)
-
-    def _find_region(self, local_directory):
-        local_directory = local_directory.as_posix().split('/')
-        if len(local_directory) > 1:
-            if local_directory[0] != 'global':
-                return local_directory[0]
-
-        if 'region' in self.account_conf:
-            return self.account_conf.get('region')
-        if 'region' in self.common_conf:
-            return self.common_conf.get('region')
-        if 'region_primary' in self.account_conf:
-            return self.account_conf.get('region_primary')
-        if 'region_primary' in self.common_conf:
-            return self.common_conf.get('region_primary')
-
-        return None
-
     def set_backend_key(self, skip_validation=False):
         # Scenarios:
         #
@@ -653,7 +655,7 @@ class TerraformContainer(LeverageContainer):
                 if not skip_validation:
                     raise KeyError()
         except (KeyError, IndexError):
-            logger.error("[red]✘[/red] Malformed [bold]config.tf[/bold] file. Missing Terraform backend bucket key.")
+            logger.error("[red]✘[/red] Malformed [bold]config.tf[/bold] file. Missing Terraform backend block. In some cases you may want to skip this check by using the --skip-validation flag, e.g. the first time you initialize a terraform-backend layer.")
             raise Exit(1)
         except Exception as e:
             logger.error("[red]✘[/red] Malformed [bold]config.tf[/bold] file. Unable to parse.")
@@ -699,4 +701,3 @@ class TFautomvContainer(TerraformContainer):
         self._prepare_container()
 
         return self._exec(command, *arguments)
-
